<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
    xmlns:func="http://www.oxygenxml.com/xsdDoc/functions" 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xlink="http://www.w3.org/1999/xlink"
    
    xmlns="http://docbook.org/ns/docbook"    
    xpath-default-namespace="http://www.oxygenxml.com/ns/doc/schema-internal" 
    exclude-result-prefixes="#all">
    
    <xsl:param name="mainFile" required="yes"/>

    
    <xsl:output method="xml" encoding="UTF-8" version="1.0"
        exclude-result-prefixes="#all"/>
    <xsl:variable name="extension">
        <xsl:variable name="ext" select="func:substring-after-last($mainFile, '.')"/>
        <xsl:choose>
            <xsl:when test="string-length($ext) = 0">
                <xsl:text></xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="concat('.', $ext)"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <xsl:variable name="chunkValueLocation">location</xsl:variable>
    <xsl:variable name="chunkValueNamespace">namespace</xsl:variable>
    <xsl:variable name="chunkValueNone">none</xsl:variable>
    
    <xsl:variable name="splitInfo" select="/schemaDoc/splitInfo"></xsl:variable>

    <!-- When all the  components are in the same file (no split after some criteria) we will use FRAMES. So the index will be redirected in a $indexFile
and the components in a $mainFile. The output file will only contain the FRAMESET-->
    <xsl:variable name="isChunkMode" as="xs:boolean" xmlns:xs="http://www.w3.org/2001/XMLSchema">
        <xsl:choose>
            <xsl:when test="compare(/schemaDoc/splitInfo/@criteria, 'none') = 0"> false </xsl:when>
            <xsl:otherwise> true </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <!-- When  NO CHUNKS we will generate a frame html. The index in one file and the content in other-->
    <xsl:variable name="indexFile" select="concat(func:substring-before-last($splitInfo/@indexLocation, '.xml'), $extension)"/>

    <xsl:variable name="mainFrame">mainFrame</xsl:variable>
    <xsl:variable name="indexFrame">indexFrame</xsl:variable>

    <!-- Traget for all the links. If we are using a FRAME representation of the html we need to specify which frame the reference will be opened in -->
    <xsl:variable name="target">
        <xsl:choose>
            <xsl:when test="boolean($isChunkMode)">
                <xsl:value-of select="$mainFrame"/>
            </xsl:when>
            <xsl:otherwise>_self</xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <!-- Used to construct an id for identifying a property of  a component. This prefix will be added to the unique component id-->
    <xsl:variable name="idsPrefixMap">
        <entry key="properties">properties_</entry>
        <entry key="usedBy">usedBy_</entry>
        <entry key="attributes">attributes_</entry>
        <entry key="children">children_</entry>
        <entry key="source">source_</entry>
        <entry key="instance">instance_</entry>
        <entry key="facets">facets_</entry>
        <entry key="diagram">diagram_</entry>
        <entry key="annotations">annotations_</entry>
        <entry key="constraints">identityConstraints_</entry>
    </xsl:variable>

    <xsl:variable name="buttonPrefix">button_</xsl:variable>

    <xsl:function name="func:getDivId" as="xs:string" xmlns:xs="http://www.w3.org/2001/XMLSchema">
        <xsl:param name="node"/>
        <xsl:value-of
            select="concat($idsPrefixMap/*[@key=local-name($node)]/text(), $node/parent::node()/@id)"
        />
    </xsl:function>

    <xsl:function name="func:getButtonId" as="xs:string" xmlns:xs="http://www.w3.org/2001/XMLSchema">
        <xsl:param name="node"/>
        <xsl:value-of select="concat($buttonPrefix/text() , func:getDivId($node))"/>
    </xsl:function>


    <xsl:variable name="schemaTypeLabels">
        <entry key="main">Main schema</entry>
        <entry key="included">Included schema</entry>
        <entry key="imported">Imported schema</entry>
    </xsl:variable>

    <xsl:variable name="componentTypeLabels">
        <entry key="Element"            xsd_name="element">Element</entry>
        <entry key="Attribute"          xsd_name="attribute">Attribute</entry>
        <entry key="Complex_Type"       xsd_name="complexType">Complex Type</entry>
        <entry key="Element_Group"      xsd_name="elementGroup">Element Group</entry>
        <entry key="Attribute_Group"    xsd_name="attributeGroup">Attribute Group</entry>
        <entry key="Simple_Type"        xsd_name="simpleType">Simple Type</entry>
        <entry key="Schema"             xsd_name="schema">Schema</entry>
        <entry key="Notation"           xsd_name="notation">Notation</entry>
    </xsl:variable>

    <!-- Build a title message 
        - If the documentation was splited by namespace we present something like: "Documentation for namespace 'ns'"
        - If the documentation was splited by location we present somehing like: "Documentation for 'Schema.xsd'"
        - If no split we always present: "Documentation for 'MainSchema.xsd'" and this function will not be used
        -->
    <xsl:function name="func:getTitle" as="xs:string" xmlns:xs="http://www.w3.org/2001/XMLSchema">
        <xsl:param name="ref"/>
        <xsl:param name="criteria"/>
        <xsl:variable name="message">
            <xsl:text>updatePageTitle('</xsl:text>
            <xsl:choose>
                <xsl:when test="compare($criteria, $chunkValueLocation) = 0">
                    <!-- The split is done after the location -->
                    <xsl:value-of select="func:getLocationChunkTitle($ref/@schemaLocation)"/>
                </xsl:when>
                <xsl:when test="compare($criteria, $chunkValueNamespace) = 0">
                    <!-- The split is done after the namespace -->
                    <xsl:value-of select="func:getNamespaceChunkTitle($ref/@ns)"/>
                </xsl:when>
            </xsl:choose>
            <xsl:text>')</xsl:text>
        </xsl:variable>
        <xsl:value-of select="$message"/>
    </xsl:function>

    <xsl:function name="func:getNamespaceChunkTitle" as="xs:string"
        xmlns:xs="http://www.w3.org/2001/XMLSchema">
        <xsl:param name="ns"/>
        <xsl:variable name="toReturn">
            <xsl:text>Schema documentation for namespace </xsl:text>
            <xsl:value-of select="$ns"/>
        </xsl:variable>
        <xsl:value-of select="$toReturn"/>
    </xsl:function>

    <xsl:function name="func:getLocationChunkTitle" as="xs:string"
        xmlns:xs="http://www.w3.org/2001/XMLSchema">
        <xsl:param name="location"/>
        <xsl:variable name="toReturn">
            <xsl:text>Schema documentation for </xsl:text>
            <xsl:value-of select="$location"/>            
        </xsl:variable>
        <xsl:value-of select="$toReturn"/>
    </xsl:function>

    <!-- Get the title of the html page by analyzing the splitInfo element -->
    <xsl:function name="func:getTitleFromSplitInfo" as="xs:string"
        xmlns:xs="http://www.w3.org/2001/XMLSchema">
        <xsl:param name="splitInfo"/>
        <xsl:choose>
            <xsl:when test="compare($splitInfo/@criteria, $chunkValueNamespace) = 0">
                <xsl:value-of select="func:getNamespaceChunkTitle($splitInfo/@value)"/>
            </xsl:when>
            <xsl:when test="compare($splitInfo/@criteria, $chunkValueLocation) = 0">
                <xsl:value-of select="func:getLocationChunkTitle($splitInfo/parent::node()/schema[compare(./schemaLocation/text(), $splitInfo/@value) = 0]/qname)"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="func:getLocationChunkTitle($splitInfo/parent::node()/schema[compare(@type, 'main') = 0]/qname)"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <!-- Get the substring before the last occurence of the given substring -->
    <xsl:function name="func:substring-before-last" as="xs:string">
        <xsl:param name="string"/>
        <xsl:param name="searched"/>
        <xsl:variable name="toReturn">
            <xsl:choose>
                <xsl:when test="contains($string, $searched)">
                    <xsl:variable name="before"
                        select="substring-before($string, $searched)"/>
                    
                    <xsl:variable name="rec" 
                        select="func:substring-before-last(substring-after($string, $searched), $searched)"/>
                    <xsl:choose>
                        <xsl:when test="string-length($rec) = 0">
                            <xsl:value-of select="$before"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="concat($before, $searched, $rec)"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:otherwise></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:value-of select="$toReturn"/>
    </xsl:function>
    
    <!-- Get the substring after the last occurence of the given substring -->
    <xsl:function name="func:substring-after-last" as="xs:string">
        <xsl:param name="string"/>
        <xsl:param name="searched"/>
        <xsl:variable name="toReturn">
            <xsl:choose>
                <xsl:when test="contains($string, $searched)">
                    <xsl:variable name="after"
                        select="substring-after($string, $searched)"/>
                    
                    <xsl:variable name="rec" 
                        select="func:substring-after-last($after, $searched)"/>
                    <xsl:choose>
                        <xsl:when test="string-length($rec) = 0">
                            <xsl:value-of select="$after"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$rec"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:otherwise></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:value-of select="$toReturn"/>
    </xsl:function>
    
    <!-- The entry point -->
    <xsl:template match="schemaDoc">
        
        <xsl:processing-instruction name="oxygen">RNGSchema="http://www.oasis-open.org/docbook/xml/5.0/rng/docbook.rng" type="xml"</xsl:processing-instruction>
        <article
            xmlns:xlink="http://www.w3.org/1999/xlink" version="5.0">
            <title>
                <xsl:value-of select="func:getTitleFromSplitInfo(./splitInfo)"/>
            </title>
            <info>
                <pubdate><xsl:value-of select="format-date(current-date(),'[D] [Mn] [Y]')"/></pubdate>
            </info>
            <xsl:call-template name="main"/>            
        </article>
    </xsl:template>

    <!-- Create the a link element to a component-->
    <xsl:template name="reference">
        <xsl:param name="ref" select="."/>
        <xsl:choose>
            <xsl:when test="exists($ref/@refId)">
                    <link linkend="{$ref/@refId}">
                        <xsl:value-of select="$ref/text()"/>
                    </link>
                    <!-- 
                    <a href="{concat(substring-before($ref/@base,'.xml'), $extension, '#', $ref/@refId)}"
                        target="{$target}">
                        <xsl:attribute name="title">
                            <xsl:choose>
                                <xsl:when test="compare('', $ref/@ns) = 0">No namespace</xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="$ref/@ns"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:attribute>
                        <xsl:variable name="criteria" select="$splitInfo/@criteria"/>
                        <xsl:if test="compare($criteria, $chunkValueNone) != 0">
                            <xsl:attribute name="onclick" select="func:getTitle($ref, $criteria)"/>
                        </xsl:if>
                        <xsl:value-of select="$ref/text()"/>
                    </a>
                    -->
            </xsl:when>
            <xsl:otherwise>
                    <xsl:copy-of select="$ref/text()"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="schema">
        <xsl:call-template name="component">
            <xsl:with-param name="type">
             <xsl:variable name="currentSchema" select="."/>
                <xsl:value-of select="$schemaTypeLabels/*[@key=$currentSchema/@type]"/>
            </xsl:with-param>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="element">
        <xsl:call-template name="component">
            <xsl:with-param name="type">Element</xsl:with-param>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="complexType">
        <xsl:call-template name="component">
            <xsl:with-param name="type">Complex Type</xsl:with-param>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="simpleType">
        <xsl:call-template name="component">
            <xsl:with-param name="type">Simple Type</xsl:with-param>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="attribute">
        <xsl:call-template name="component">
            <xsl:with-param name="type">Attribute</xsl:with-param>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="attributeGroup">
        <xsl:call-template name="component">
            <xsl:with-param name="type">Attribute Group</xsl:with-param>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="elementGroup">
        <xsl:call-template name="component">
            <xsl:with-param name="type">Element Group</xsl:with-param>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="notation">
        <xsl:call-template name="component">
            <xsl:with-param name="type">Notation</xsl:with-param>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="facets">
        <xsl:variable name="boxID" select="func:getDivId(.)"/>
        <row>
            <entry>
                 Facets
            </entry>
            <entry>
                <informaltable frame="none" colsep="0" >      
                    <tgroup cols="3">
                        <colspec colwidth="2*" align="left"/>
                        <colspec colwidth="3*" align="left"/>
                        <colspec colwidth="5*" align="left"/>
                        <tbody>
                    <xsl:for-each select="./facet">
                        <row>
                            <entry>
                                <xsl:value-of select="@name"/>
                            </entry>
                            <entry>
                                <literal>
                                    <xsl:value-of select="@value"/>
                                </literal>
                            </entry>
                            <entry>
                                <xsl:for-each select="annotation">
                                    <xsl:call-template name="buildAnnotation"/>                                        
                                </xsl:for-each>
                            </entry>
                        </row>
                    </xsl:for-each>
                            </tbody>
                    </tgroup>
                </informaltable>
            </entry>
        </row>
    </xsl:template>

    <xsl:template match="properties">
        <xsl:variable name="boxID" select="func:getDivId(.)"/>
        <row>
            <entry>
                    Properties
            </entry>
            <entry>
                    <informaltable frame="none" colsep="0">
                        <tgroup cols="2">                            
                            <colspec colwidth="2*" align="left"/>
                            <colspec colwidth="8*" align="left"/>
                            <tbody>
                            <xsl:for-each select="./property">
                                <row>
                                    <entry>
                                        <xsl:value-of select="name"/>:
                                    </entry>
                                    <entry>
                                        <literal>
                                            <xsl:value-of select="value"/>
                                        </literal>
                                    </entry>
                                </row>
                            </xsl:for-each>
                            </tbody>
                        </tgroup>
                    </informaltable>
            </entry>
        </row>
    </xsl:template>

    <xsl:template match="namespace">
        <row>
            <entry>
                Namespace
            </entry>
            <entry>
                <xsl:choose>
                    <xsl:when test="compare('', text()) != 0">
                        <xsl:value-of select="text()"/>
                    </xsl:when>
                    <xsl:otherwise>No namespace</xsl:otherwise>
                </xsl:choose>
            </entry>
        </row>
    </xsl:template>

    <xsl:template match="schemaLocation">
        <row>
            <entry>
                Schema location
            </entry>
            <entry>
                <xsl:value-of select="text()"/>
            </entry>
        </row>
    </xsl:template>
    
    <xsl:template match="diagram">
        <xsl:variable name="boxID" select="func:getDivId(.)"/>
        <xsl:variable name="width" select="@width div 2.0"/>
        <xsl:variable name="widthLimited">
            <xsl:choose>
                <xsl:when test="$width > 350">350</xsl:when>
                <xsl:otherwise><xsl:value-of select="$width"/></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>        
        <xsl:variable name="width" select="concat($widthLimited,'pt')"></xsl:variable>
        <row>
            <entry>
                Diagram
            </entry>
            <entry>
                <xsl:variable name="hasMap" as="xs:boolean" select="count(map) != 0"/>                
                <para>
                    <xsl:choose>
                        <xsl:when test="map/area" xml:space="preserve">
                            
                            <mediaobject>  
                                <imageobjectco>  
                                    <xsl:variable name="mapName" select="map/@name"/>
                                    <areaspec xml:id="{$mapName}">  
                                        <xsl:for-each select="map/area">  
                                            <area 
                                                linkends="{substring-after(@href,'.tmp#')}"  
                                                units="other"
                                                otherunits="imagemap"  
                                                coords="{@coords}"                                    
                                                xml:id="{concat($mapName,'-',position())}"/>
                                        </xsl:for-each>
                                        
                                    </areaspec>
                                    <imageobject>                                        
                                                <imagedata fileref="{location/text()}" width="{$width}"/>
                                        <xsl:comment>If you want to use this Docbook document to generate
                                            HTML output you will have to delete the "width" attributes in order for the imagemaps to function.
                                        </xsl:comment>
                                    </imageobject>                                    
                                </imageobjectco>
                            </mediaobject>
                            
                        </xsl:when>
                        <xsl:otherwise>
                            <mediaobject>
                                <imageobject>
                                    <imagedata fileref="{location/text()}" width="{$width}"/>
                                </imageobject>                                
                            </mediaobject>
                        </xsl:otherwise>
                    </xsl:choose>
                </para>
            </entry>
        </row>
    </xsl:template>
    
    <xsl:template match="usedBy">
        <xsl:variable name="boxID" select="func:getDivId(.)"/>
        <row>
            <entry>
                    Used by
            </entry>
            <entry>
                
                    <informaltable frame="none" colsep="0">
                        <tgroup cols="2">
                        <colspec colwidth="2*" align="left"/>
                        <colspec colwidth="8*" align="left"/>
                        <tbody>
                        <xsl:for-each-group select="./ref" group-by="@refType">
                            <row>
                                <entry>
                                    <xsl:variable name="currentRef" select="."/>
                                    <xsl:value-of
                                        select="$componentTypeLabels/*[@key=$currentRef/@refType]"/>
                                    <xsl:if test="count(current-group()) > 1">
                                        <xsl:text>s</xsl:text>
                                    </xsl:if>
                                    <xsl:text> </xsl:text>
                                </entry>
                                <entry>
                                    <xsl:for-each select="current-group()">
                                        <xsl:sort select="text()"/>
                                        <xsl:call-template name="reference"/>
                                        <xsl:if test="position() != last()">
                                            <xsl:text>, </xsl:text>
                                        </xsl:if>
                                    </xsl:for-each>
                                </entry>
                            </row>
                        </xsl:for-each-group>
                        </tbody>
                        </tgroup>
                    </informaltable>
            </entry>
        </row>
    </xsl:template>

    <xsl:template match="attributes">
        <xsl:variable name="boxID" select="func:getDivId(.)"/>
        <row>
            <entry>
                    Attributes
            </entry>
            <entry>
                
                <informaltable frame="none" colsep="1">            
                    <tgroup cols='5'>
                        <colspec colnum="1" colname="col1" align="left" colwidth="2*"/>
                        <colspec colnum="2" colname="col2" align="left" colwidth="2*"/>
                        <colspec colnum="3" colname="col3" align="left" colwidth="2*"/>
                        <colspec colnum="4" colname="col4" align="left" colwidth="2*"/>
                        <colspec colnum="5" colname="col5" align="left" colwidth="2*"/>
                        <thead>
                            <row>
                                <entry>QName</entry>
                                <entry>Type</entry>
                                <entry>Fixed</entry>
                                <entry>Default</entry>
                                <entry>Use</entry>
                            </row>                            
                        </thead>
                        <tbody>
                        <xsl:for-each select="attr">
                            <xsl:sort select="ref/text()"/>
                            <row>
                                <entry>
<!--                                    <xsl:processing-instruction name="dbhtml"> bgcolor="#EEEEEE" </xsl:processing-instruction>
                                    <xsl:processing-instruction name="dbfo"> bgcolor="#EEEEEE" </xsl:processing-instruction>
-->                                 <emphasis role="bold">   
                                        <xsl:call-template name="reference">
                                            <xsl:with-param name="ref" select="ref"/>
                                        </xsl:call-template>
                                    </emphasis>
                                </entry>
                                <entry>
                                    <xsl:call-template name="typeEmitter">
                                        <xsl:with-param name="type" select="type"/>
                                    </xsl:call-template>
                                </entry>
                                <entry>
                                    <xsl:value-of select="fixed"/>
                                </entry>
                                <entry>
                                    <xsl:value-of select="default"/>
                                </entry>
                                <entry>
                                    <xsl:value-of select="use"/>
                                </entry>                        
                            </row>
                            <xsl:if test="annotations/annotation">                                
                                <row>                                
                                    <entry  namest="col2" nameend="col5">
                                        <xsl:for-each select="annotations/annotation">
                                            <xsl:call-template name="buildAnnotation"/>                                       
                                        </xsl:for-each>                                        
                                    </entry>
                                </row>
                            </xsl:if>
                        </xsl:for-each>                           
                        </tbody>
                    </tgroup>
                </informaltable>
                
            </entry>
        </row>
    </xsl:template>

    <xsl:template name="component">
        <xsl:param name="type"/>
           <sect3>
               <xsl:attribute name="xml:id" select="@id"/>
               <title>                   
                   <xsl:choose>
                       <xsl:when test="exists(redefinedComponent)">
                           <xsl:text>redefinition of </xsl:text>
                           <xsl:value-of select="$type"/>
                           <xsl:text> </xsl:text>
                           <xsl:call-template name="reference">
                               <xsl:with-param name="ref" select="redefinedComponent"/>
                           </xsl:call-template>
                       </xsl:when>
                       <xsl:otherwise>                    
                           <xsl:value-of select="$type"/>
                           <xsl:text> </xsl:text>                        
                           <literal>
                               <xsl:for-each select="declarationPath/ref">
                                   <xsl:value-of select="."/>                                 
                                     <!-- Placed in the title, the links generated by the reference 
                                         template are breaking the TOC, adding the text 
                                         "the section called" before each link target. -->
                                   
<!--                                   <xsl:call-template name="reference"/> -->
                                   <xsl:text> / </xsl:text>
                               </xsl:for-each>
                               <xsl:if test="compare(local-name(.), 'attribute') = 0">
                                   <xsl:text>@</xsl:text>
                               </xsl:if>
                               <xsl:value-of select="qname/text()"/>
                           </literal>
                       </xsl:otherwise>
                   </xsl:choose>                      
               </title>
            <informaltable frame="all" colsep="1">
                <tgroup cols="2">
                <colspec colwidth="2*" align="left"/>
                <colspec colwidth="8*" align="left"/>
                <tbody>
                    <xsl:apply-templates select="namespace | annotations"></xsl:apply-templates>
                    <xsl:apply-templates select="diagram | type | typeHierarchy | properties"></xsl:apply-templates>
                    <xsl:apply-templates select="facets"/>
                    <xsl:apply-templates select="substitutionGroup | substitutionGroupAffiliation"/>
                    <xsl:apply-templates select="usedBy | model | children | attributes | contraints | instance | source"></xsl:apply-templates>
                    <xsl:apply-templates select="publicid | systemid"/>
                    <xsl:apply-templates select="schemaLocation"/>
                </tbody>         
                </tgroup>
            </informaltable>
           </sect3>              
    </xsl:template>

    <xsl:template name="typeEmitter">
        <xsl:param name="type"/>
        <xsl:for-each select="$type/node()">
            <xsl:choose>
                <xsl:when test="compare('ref', local-name()) = 0">
                    <xsl:call-template name="reference">
                        <xsl:with-param name="ref" select="."/>
                    </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="."/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="element/type | complexType/type | attribute/type | simpleType/type">
        <row>
            <entry>
                Type
            </entry>
            <entry>
                <xsl:call-template name="typeEmitter">
                    <xsl:with-param name="type" select="."/>
                </xsl:call-template>
            </entry>
        </row>
    </xsl:template>

    <!-- Show the hierarchy type  -->
    <xsl:template name="hierarchyOutput">
        <xsl:param name="refs"/>
        <xsl:param name="index" as="xs:integer" select="1"
            xmlns:xs="http://www.w3.org/2001/XMLSchema"/>
        <itemizedlist>
            <listitem>
                <para>
                    <xsl:call-template name="reference">
                        <xsl:with-param name="ref" select="$refs[$index]"/>
                    </xsl:call-template>
                </para>
                <xsl:if test="$index &lt; count($refs)">
                    <xsl:call-template name="hierarchyOutput">
                        <xsl:with-param name="refs" select="$refs"/>
                        <xsl:with-param name="index" select="$index + 1"/>
                    </xsl:call-template>
                </xsl:if>
            </listitem>
        </itemizedlist>
    </xsl:template>

    <xsl:template match="typeHierarchy">
        <row>
            <entry>
                Type hierarchy
            </entry>
            <entry>
                <xsl:call-template name="hierarchyOutput">
                    <xsl:with-param name="refs" select="ref"/>
                </xsl:call-template>
            </entry>
        </row>
    </xsl:template>

    <xsl:template match="model">
        <row>
            <entry>
                Model
            </entry>
            <entry>
                <xsl:call-template name="groupTemplate">
                    <xsl:with-param name="group" select="group[1]"/>
                </xsl:call-template>
            </entry>
        </row>
    </xsl:template>

    <xsl:template name="groupTemplate">
        <xsl:param name="group" select="."/>
        <xsl:variable name="compositor">
            <xsl:value-of select="$group/@compositor"/>
        </xsl:variable>
        <xsl:variable name="separator">
            <xsl:if test="compare($compositor, 'sequence') = 0">
                <xsl:text> , </xsl:text>
            </xsl:if>
            <xsl:if test="compare($compositor, 'choice') = 0">
                <xsl:text> | </xsl:text>
            </xsl:if>
            <xsl:if test="compare($compositor, 'all') = 0">
                <xsl:text> </xsl:text>
            </xsl:if>
        </xsl:variable>

        <xsl:if test="compare($compositor, 'all') = 0">
            <xsl:text>ALL(</xsl:text>
        </xsl:if>
        <xsl:for-each
            select="$group/*[compare(local-name(.), 'group') = 0 or compare(local-name(.), 'ref') = 0]">
            <xsl:if test="position() != 1">
                <xsl:value-of select="$separator"/>
            </xsl:if>
            <xsl:choose>
                <xsl:when test="compare(local-name(.), 'ref') = 0">
                    <xsl:call-template name="reference"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:variable name="nextCompositor">
                        <xsl:value-of select="@compositor"/>
                    </xsl:variable>

                    <xsl:if test="compare($compositor, $nextCompositor) != 0">
                        <xsl:text>(</xsl:text>
                    </xsl:if>

                    <xsl:call-template name="groupTemplate"/>

                    <xsl:if test="compare($compositor, $nextCompositor) != 0">
                        <xsl:text>)</xsl:text>
                    </xsl:if>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>

        <xsl:if test="compare($compositor, 'all') = 0">
            <xsl:text>)</xsl:text>
        </xsl:if>
    </xsl:template>

    <xsl:template match="children">
        <row>
            <entry>
                Children
            </entry>
            <entry>
                <xsl:for-each select="child">
                    <xsl:sort select="ref/text()"/>
                    <xsl:call-template name="reference">
                        <xsl:with-param name="ref" select="ref"/>
                    </xsl:call-template>
                    <xsl:if test="position() != last()">
                        <xsl:text>, </xsl:text>
                    </xsl:if>
                </xsl:for-each>
            </entry>
        </row>
    </xsl:template>

    <xsl:template match="source | instance">
        <xsl:variable name="boxID" select="func:getDivId(.)"/>
        <row>
            <entry>
                <literal>
                    <xsl:choose>
                        <xsl:when test="compare(local-name(.), 'source') = 0">Source</xsl:when>
                        <xsl:otherwise>Instance</xsl:otherwise>
                    </xsl:choose>
                 </literal>
            </entry>
            <entry>
            <!-- Formats an XML source section-->
            <xsl:variable name="tokens" select="token"/>
            <xsl:call-template name="formatXmlSource">
                <xsl:with-param name="tokens" select="$tokens"/>
            </xsl:call-template>
            </entry>
        </row>
    </xsl:template>


    <xsl:template match="constraints">
        <xsl:variable name="boxID" select="func:getDivId(.)"/>
        <row>
            <entry>
                    Identity constraints
            </entry>
            <entry>                
                    <informaltable frame="none" colsep="1">   
                        <tgroup cols="5">
                            <colspec colwidth="2*" align="left"/>
                            <colspec colwidth="2*" align="left"/>
                            <colspec colwidth="2*" align="left"/>
                            <colspec colwidth="2*" align="left"/>
                            <colspec colwidth="2*" align="left"/>
                            <thead>
                                <row>
                                    <entry>QName</entry>
                                    <entry>Type</entry>
                                    <entry>Refer</entry>
                                    <entry>Selector</entry>
                                    <entry>Field(s)</entry>
                                </row>
                            </thead>
                            <tbody>
                            <xsl:for-each select="constraint">
                                <row>
                                    <entry>
                                        <emphasis role="bold">
                                            <xsl:value-of select="name"/>
                                        </emphasis>
                                    </entry>
                                    <entry>
                                        <xsl:value-of select="type"/>
                                    </entry>
                                    <entry>
                                        <xsl:value-of select="refer"/>
                                    </entry>
                                    <entry>
                                        <xsl:value-of select="selector"/>
                                    </entry>
                                    <entry>
                                        <xsl:value-of select="fields"/>
                                    </entry>
                                </row>
                            </xsl:for-each>
                            </tbody>
                        </tgroup>
                    </informaltable>
                                                
            </entry>
        </row>
    </xsl:template>

    <xsl:template match="publicid | systemid">
        <row>
            <entry>
                <literal>
                    <xsl:choose>
                        <xsl:when test="compare(local-name(.), 'publicid') = 0"
                            >Public ID</xsl:when>
                        <xsl:otherwise>System ID</xsl:otherwise>
                    </xsl:choose>
                </literal>
            </entry>
            <entry><xsl:value-of select="text()"/></entry>
        </row>
    </xsl:template>

    <xsl:template match="substitutionGroup | substitutionGroupAffiliation">
        <row>
            <entry>
                <literal>
                    <xsl:choose>
                        <xsl:when test="compare(local-name(.), 'substitutionGroup') = 0"
                            >Substitution Group</xsl:when>
                        <xsl:otherwise>Substitution Group Affiliation</xsl:otherwise>
                    </xsl:choose>
                </literal>
            </entry>
            <entry>
                
                <itemizedlist>
                    <xsl:for-each select="ref">
                        <listitem>
                            <xsl:call-template name="reference"/>
                        </listitem>
                    </xsl:for-each>
                </itemizedlist>
            </entry>
        </row>
    </xsl:template>

    <xsl:template match="annotations">
        <xsl:variable name="boxID" select="func:getDivId(.)"/>
        <row>
            <entry>
                Annotations
            </entry>
            <entry>            
                <para>
                    <xsl:for-each select="annotation">
                           <xsl:call-template name="buildAnnotation"/>                            
                    </xsl:for-each>
                </para>
            </entry>
        </row>
    </xsl:template>

    <!-- Builds an annotation representation from the context annotation -->
    <xsl:template name="buildAnnotation">
            <xsl:if test="exists(@source)">
                <link xlink:href="{@source}"><xsl:value-of select="@source"/></link>
            </xsl:if>
            <xsl:variable name="tokens" select="token"/>
            <xsl:choose>
                <xsl:when test="empty($tokens)">
                    <xsl:for-each select="child::node()">
                        <xsl:copy-of select="."/>
                    </xsl:for-each>
                </xsl:when>
                <xsl:otherwise>
                    <!-- Formats an XML source section-->
                    <xsl:call-template name="formatXmlSource">
                        <xsl:with-param name="tokens" select="$tokens"/>
                    </xsl:call-template>
                </xsl:otherwise>
            </xsl:choose>
    </xsl:template>

    <xsl:template name="formatXmlSource">
        <xsl:param name="tokens"/>
        
        <programlisting>
        <xsl:for-each select="$tokens">
        <xsl:choose>
            <!-- Simple text. -->
            <xsl:when test="@type = 'tokenText' or @type = 'tokenCDATA'">
                <xsl:choose>
                    <xsl:when test="text() = ' '">
                        <!-- Just a whitespace should preserve it, 
                            may be it dellimits something.  -->
                        <xsl:text xml:space="preserve"> </xsl:text>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="normalize-space(text())"/>                                                
                    </xsl:otherwise>
                </xsl:choose>                                        
            </xsl:when>
            <!-- The indent -->
            <xsl:when test="@type = 'tokenIndent'">
                <xsl:value-of select="text()"/>
            </xsl:when>
            <!-- Other tokens -->
            <xsl:otherwise>
                <xsl:variable name="class">
                    <xsl:choose>
                        <xsl:when test="@type = 'tokenElement'">element</xsl:when>
                        <xsl:when test="@type = 'tokenAttrName'">attribute</xsl:when>
                        <xsl:when test="@type = 'tokenAttrValue'">attvalue</xsl:when>
                        <xsl:when test="@type = 'tokenComment'">comment</xsl:when>
                        <xsl:when test="@type = 'tokenPI'">xmlpi</xsl:when>
                        <xsl:when test="@type = 'tokenEntity'">genentity</xsl:when>                        
                    </xsl:choose>                    
                </xsl:variable>
                <tag class="{normalize-space($class)}"><xsl:value-of select="text()"/></tag>                                           
            </xsl:otherwise>
        </xsl:choose>            
        </xsl:for-each>
        </programlisting>
    </xsl:template>
    
    <xsl:template name="main">            
        <xsl:for-each-group select="element | complexType | attribute | simpleType | elementGroup | schema | attributeGroup | notation" group-by="namespace">
            <sect1>
                <title> Namespace: "<xsl:value-of select="namespace"/>" </title>                
                    <xsl:for-each-group select="current-group()"  group-by="node-name(.)">
                        <sect2>
                            <xsl:variable name="name" select="node-name(.)"/>
                            <title><xsl:value-of select="$componentTypeLabels/*[@xsd_name=string($name)]"/>s</title>
                            <xsl:for-each select="current-group()">                            
                                <xsl:apply-templates select="."/>
                            </xsl:for-each>
                        </sect2>
                    </xsl:for-each-group>
            </sect1>                                
        </xsl:for-each-group>
                
    </xsl:template>

    <xsl:template match="text()"/>
</xsl:stylesheet>
