/*
 *  The Syncro Soft SRL License
 *
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  3. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  4. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  5. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.sample.plugin.converter;

import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;

import javax.swing.JTextField;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.log4j.Logger;

import ro.sync.exml.plugin.PluginManager;

/**
 *  Description of the Class
 *
 *@author     radu
 *@created    October 16, 2002
 *@version    $Revision: 1.14 $
 */
public class ConversionDialogTest extends TestCase {
    /**
     *  Logger for logging.
     */
    private static Logger logger = Logger.getLogger(ConversionDialogTest.class.getName());

    /** @link dependency 
     * @stereotype tests*/
    /*#ConversionDialog lnkConversionDialog;*/

    /**
     *  Constructor for the ConversionDialogTest object
     *
     *@param  testName  Description of Parameter
     */
    public ConversionDialogTest(String testName) {
        super(testName);
    }


    /**
     *  A unit test suite for JUnit
     *
     *@return    The test suite
     */
    public static Test suite() {
        return new TestSuite(ConversionDialogTest.class);
    }


    /**
     *  The main program for the ConversionDialogTest class
     *
     *@param  args  The command line arguments
     */
    public static void main(String[] args) {
        ConversionDialogTest test = new ConversionDialogTest("test");
        test.testCopy();
        test.testExtractNumberStr();
        test.testFormatNumberWithLabel();
    }


    /**
     *  A unit test for JUnit
     */
    public void testCopy() {
        try {
            ConversionDialog cd = new ConversionDialog(null, "cd");

            // Set s1 text field text
            cd.masterValueTextField = new JTextField("123");
            // Copy text of s 1 text field to clipboard
            cd.copyMasterValue();
            // Test result
            logger.debug("Paste");
            Clipboard clipboard = cd.getToolkit().getSystemClipboard();
            logger.debug("Getting content.");
            Transferable content = clipboard.getContents(this);
            logger.debug("Obtaining transfer data");
            String dstData = (String) (content.getTransferData(DataFlavor.stringFlavor));
            if(logger.isDebugEnabled()){
              logger.debug("Content is : " + dstData);
            }
            assertEquals("123", dstData);
            logger.debug("Test copy ok.");
        } catch (Exception ex) {
            logger.error(ex, ex);
            fail(ex.getMessage());
        }
    }


    /**
     *  A unit test for JUnit
     */
    public void testExtractNumberStr() {
        try {
            ConversionDialog cd = new ConversionDialog(null, "cd");
            assertEquals("1.22", cd.extractNumberStr("1.22 mm"));
            assertEquals("1.23", cd.extractNumberStr("1.23"));
        } catch (Exception ex) {
            logger.error(ex, ex);
            fail(ex.getMessage());
        }
    }


    /**
     *  A unit test for JUnit
     */
    public void testFormatNumberWithLabel() {
        try {
            ConversionDialog cd = new ConversionDialog(null, "cd");
            cd.includeLabelCheck = new javax.swing.JCheckBox("", true);
            assertEquals("1.22 mm", cd.formatNumberWithLabel(""+1.22, "mm"));
        } catch (Exception ex) {
            logger.error(ex, ex);
            fail(ex.getMessage());
        }
    }


    /**
     *  The JUnit setup method
     */
    protected void setUp() {
      PluginManager.getInstance(); 
    }


    /**
     *  The teardown method for JUnit
     */
    protected void tearDown() { 
      // Do nothing.
    }

}
