/*
 *  The Syncro Soft SRL License
 *
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  3. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  4. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  5. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.sample.plugin.converter;

/**
 *  Defines the abstract conversion entry.
 *
 *@author     dan, radu
 *@created    October 14, 2002
 *@version    $Revision: 1.16 $
 */
public abstract class ConversionEntry {

    /**
     *  Constant for defining that all decimals should be included in the
     *  result.
     */
    public final static int ALL_DECIMALS = -1;

    /**
     *  The slave measure label.
     */
    protected String slaveLabel;

    /**
     *  The master measure label.
     */
    protected String masterLabel;

    /**
     *  The slave measure value.
     */
    protected String slaveValue;

    /**
     *  The master measure label.
     */
    protected String masterValue;

    /**
     *  The decimals number to show.
     */
    protected int decimalsNumber;


    /**
     *  Return decimals number to show.
     *
     *@return    The decimals number.
     */
    public int getDecimalsNumber() {
        return decimalsNumber;
    }


    /**
     *  Return slave measure label.
     *
     *@return    The slave measure label.
     */
    public String getSlaveLabel() {
        return slaveLabel;
    }


    /**
     *  Return master measure label.
     *
     *@return    The master measure label.
     */
    public String getMasterLabel() {
        return masterLabel;
    }


    /**
     *  Return slave measure value.
     *
     *@return    The slave measure value.
     */
    public String getSlaveValue() {
        return slaveValue;
    }


    /**
     *  Return master measure value.
     *
     *@return    The master measure value.
     */
    public String getMasterValue() {
        return masterValue;
    }


    /**
     *  Converts the argument from the master measure to the slave measure if
     *  direct attribute is true, else reverse.
     *
     *@param  what            The conversion argument.
     *@param  direct          If true, then make the conversion from master to
     *      slave, otherwise reverse.
     *@param  decimalsNumber  The decimals number witch be included in the
     *      result.
     *@return                 The conversion result.
     */
    public abstract double convert(double what, boolean direct, int decimalsNumber);


    /**
     *  Format a double number with specified decimals number to be included in
     *  the result.
     *
     *@param  numberToFormat             Number to format.
     *@param  decimalsNumberToRepresent  The decimals number.
     *@return                            The formated number.
     */
    protected double formatDouble(double numberToFormat, int decimalsNumberToRepresent) {
        String numberStr = "" + numberToFormat;
        double result = 0;

        int decNumber = 0;
        if (decimalsNumberToRepresent == -2) {
            decNumber = this.decimalsNumber;
        } else if (decimalsNumberToRepresent == -1) {
            decNumber = ALL_DECIMALS;
        } else {
            decNumber = decimalsNumberToRepresent;
        }

        // If decimals number >= 0 then format number
        // else leave number intact
        if (decNumber != ALL_DECIMALS) {
            // Find index of '.'
            int index = numberStr.indexOf('.');

            int endIndex = index + decNumber + 1;
            if (endIndex > numberStr.length()) {
                endIndex = numberStr.length();
            }
            if (decNumber == 0) {
                endIndex = index;
            }

            String resultStr = numberStr.substring(0, endIndex);
            result = Double.parseDouble(resultStr);
        } else {
            result = numberToFormat;
        }
        return result;
    }
}
