/*
 *  The Syncro Soft SRL License
 *
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  3. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  4. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  5. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.sample.plugin.converter;

import java.util.List;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.log4j.Logger;

import ro.sync.exml.plugin.PluginManager;


/**
 *  Description of the Class
 *
 *@author     dan, radu
 *@created    October 15, 2002
 *@version    $Revision: 1.18 $
 */
public class ConversionManagerTest extends TestCase {

    /**
     *  Logger for logging.
     */
    static Logger logger = Logger.getLogger(ConversionManagerTest.class.getName());

    /** @link dependency
     * @stereotype tests*/
    /*#ConversionManager lnkConversionManager;*/

    /**
     *  Constructor for the ConversionManagerTest object
     *
     *@param  testName  Description of Parameter
     */
    public ConversionManagerTest(String testName) {
        super(testName);
    }


    /**
     *  A unit test suite for JUnit
     *
     *@return    The test suite
     */
    public static Test suite() {
        return new TestSuite(ConversionManagerTest.class);
    }


    /**
     *  The main program for the ConversionManagerTest class
     *
     *@param  args  The command line arguments
     */
    public static void main(String[] args) {
        ConversionManagerTest test = new ConversionManagerTest("test");
        test.setUp();
        test.testGetDefaultValueAsBoolean();
        test.testGetFormats();
        test.testGetStringResource();
        test.tearDown();
    }


    /**
     *  A unit test for getGormats method
     */
    public void testGetFormats() {
        try {
            ConversionManager cm = new ConversionManager();
            // Get formats
            List result = cm.getFormats();
            assertEquals("Number of formats must be ", 7, result.size());

        } catch (Exception ex) {
            logger.error(ex, ex);
            fail(ex.getMessage());
        }
    }


    /**
     *  A unit test for JUnit
     */
    public void testGetDefaultValueAsBoolean() {
        try {
            ConversionManager cm = new ConversionManager();
            // Get formats
            Boolean result = cm.getDefaultValueAsBoolean("Lock labels");
            Boolean expected = new Boolean(true);
            assertEquals(result, expected);

            result = cm.getDefaultValueAsBoolean("Lock labels kkk");
            expected = new Boolean(false);
            assertEquals(result, expected);

        } catch (Exception ex) {
            logger.error(ex, ex);
            fail(ex.getMessage());
        }
    }

    /**
     *  A unit test for JUnit
     */
    public void testGetStringResource() {
        try {
            ConversionManager cm = new ConversionManager();
            // Get formats
            String result = cm.getStringResource("Format");
            String expected = "Format";
            assertEquals(result, expected);

            result = cm.getStringResource("Master system");
            expected = "U.S. System";
            assertEquals(result, expected);

            result = cm.getStringResource("Copy value and close");
            expected = "Copy value and close";
            assertEquals(result, expected);

            result = cm.getStringResource("Include labels");
            expected = "Include labels";
            assertEquals(result, expected);

            result = cm.getStringResource("Slave system");
            expected = "Metric System";
            assertEquals(result, expected);

            result = cm.getStringResource("Swap values");
            expected = "Swap values";
            assertEquals(result, expected);

            result = cm.getStringResource("Type");
            expected = "Type";
            assertEquals(result, expected);

        } catch (Exception ex) {
            logger.error(ex, ex);
            fail(ex.getMessage());
        }
    }

    /**
     *  The JUnit setup method
     */
    protected void setUp() {
      PluginManager.getInstance();
    }


    /**
     *  The teardown method for JUnit
     */
    protected void tearDown() { 
      // Do nothing,
    }

}
