/*
 *  The Syncro Soft SRL License
 *
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  3. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  4. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  5. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.sample.plugin.converter;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.log4j.Logger;

/**
 *  Description of the Class
 *
 *@author     dan, radu
 *@created    October 15, 2002
 *@version    $Revision: 1.13 $
 */
public class FormulaConversionEntryTest extends TestCase {
    /**
     *  Logger for logging.
     */
    private static Logger logger = Logger.getLogger(FormulaConversionEntryTest.class.getName());

    /** @link dependency 
     * @stereotype tests*/
    /*#FormulaConversionEntry lnkFormulaConversionEntry;*/

    /**
     *  Constructor for the FormulaConversionEntryTest object
     *
     *@param  testName  Description of Parameter
     */
    public FormulaConversionEntryTest(String testName) {
        super(testName);
    }


    /**
     *  A unit test suite for JUnit
     *
     *@return    The test suite
     */
    public static Test suite() {
        return new TestSuite(FormulaConversionEntryTest.class);
    }


    /**
     *  A unit test for convert method
     */
    public void testConvert() {
        try {
            // Create a FormulaConversionEntry with
            //    s1Value, s2s1Value, s1Label, s2Label
            //    v1 = 10.3, v2 = 5.7, decimals = 10
            FormulaConversionEntry fce =
                    new FormulaConversionEntry("s1L", "s2L", "s1V", "s2V", 10, 10.3, 5.7);
            //  Test FormulaConversionEntry attributes
            assertEquals("s1L", fce.getMasterLabel());
            assertEquals("s2L", fce.getSlaveLabel());
            assertEquals("s1V", fce.getMasterValue());
            assertEquals("s2V", fce.getSlaveValue());
            assertEquals(10,fce.getDecimalsNumber());

            // Convert 12 Fahrenheit to Celsius
            double result = fce.convert(12, true, -1);
            assertEquals(0.298245, result, 0.00001);

            // Convert 12 Celsius to Fahrenheit
            result = fce.convert(12, false, -1);
            assertEquals(78.7, result, 0.00001);
        } catch (Exception ex) {
          if(logger.isDebugEnabled()){
            logger.debug(ex,ex);
          }
          fail(ex.getMessage());
        }
    }

    /**
     *  A unit test for FormulaConversionEntry contructor
     */
    public void testFormulaConversionEntry() {
        try {
            // Create a FormulaConversionEntry with
            //  s1Value, s2s1Value, s1Label, s2Label
            //  v1 = 10.3, v2 = 5.7, decimals = 3
            FormulaConversionEntry fce =
              new FormulaConversionEntry("s1L", "s2L", "s1V", "s2V", 3, 10.3, 5.7);

            // Create a FormulaConversionEntry with
            //  s1Label = null => throw ConversionException
            fce = null;
            try{
              fce = new FormulaConversionEntry(null, "s2L", "s1V", "s2V", 3, 10.3, 5.7);
            } catch (ConversionException ex){
                logger.debug("Error");
                if(logger.isDebugEnabled()){
                  logger.debug(ex, ex);
                }
            }
            // fce must be null
            assertNull(fce);

            // Create a FormulaConversionEntry with
            //  s1Value, s2s1Value, s1Label, s2Label
            //  v1 = 10.3, decimals = 10

            //  v2 = 0 => ConversionException
            try{
              fce = new FormulaConversionEntry("s1L", "s2L", "s1V", "s2V", 3, 10.3, 0);
            } catch (ConversionException ex){
                logger.debug("Error");
                if(logger.isDebugEnabled()){
                  logger.debug(ex, ex);
                }
            }
            // fce must be null
            assertNull(fce);
        } catch (Exception ex) {
          logger.error(ex,ex);
        }
    }


    /**
     *  The JUnit setup method
     */
    protected void setUp() { 
      // Do nothing.
    }


    /**
     *  The teardown method for JUnit
     */
    protected void tearDown() {
      // Do nothing.
    }

}
