/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.sample.plugin.openredirect;

import java.io.File;
import java.net.URL;

import ro.sync.exml.MainFrame;
import ro.sync.exml.MainFrameDescriptor;
import ro.sync.junit.JFCTestCase;
import ro.sync.udc.UsageDataCollector;
import ro.sync.ui.UiUtil;
import ro.sync.util.URLUtil;

/**
 * Tests the file2 protocol by writing the line numbers in the file indicated by the URL
 */
public class OpenRedirectPluginExtensionTest extends JFCTestCase {

  /**
   * Test open redirect.
   * 
   * @throws Exception 
   */
  public void testOpenRedirect() throws Exception {
    UsageDataCollector.ENABLED = false;
    mainFrameDescriptor = new MainFrameDescriptor();
    mainFrameDescriptor.setStandalone(true);
    final MainFrame mainFrame = new MainFrame(
        mainFrameDescriptor, 
        null, 
        true, 
        null, 
        null, 
        true,
        false,
        null);
    
    UiUtil.invokeSynchronously(new Runnable() {
      public void run() {
        mainFrame.setVisible(true);
      }
    });
    flushAWTBetter();

    mainFrame.getWorkspaceController().open(URLUtil.correct(new File("test/EXM15995/test.docx")).toString(), null);
    
    // document.xml from the archive must be opened in the editor
    URL[] allOpenedURLs = mainFrame.getWorkspaceController().getAllOpenedURLs();
    assertEquals(1, allOpenedURLs.length);
    String expected = "zip:" + URLUtil.correct(new File("test/EXM15995/test.docx")).toString() 
        + "!/word/document.xml";
    assertEquals(expected,  URLUtil.correct(allOpenedURLs[0]).toString());
    
    // The docx must be opened in the archive browser
    URL archiveRootURL = mainFrame.getWorkspaceController().getArchiveBrowser().getArchiveRootURL();
    expected =
      URLUtil.correct(new File("test/EXM15995/test.docx")).toString();
    assertEquals(expected, URLUtil.correct(archiveRootURL).toString());
  }
}
