/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.sample.plugin.customprotocol;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.net.URLConnection;

import junit.framework.TestCase;
import ro.sync.net.protocol.Installer;

/**
 * Tests the file2 protocol by writing the line numbers in the file indicated by the URL
 */
public class CustomProtocolPluginExtensionTest extends TestCase {

  /**
   * Uses the file2 protocol to write line numbers in the file indicated by the URL
   * 
   * @param url
   *          The URL of the file
   * @throws Exception
   */
  public static void rewriteFile(URL url) throws Exception {

    URLConnection connection = url.openConnection();
    // The input stream of the connection
    BufferedReader input = new BufferedReader(new InputStreamReader(connection.getInputStream()));
    // The new content of the file
    String newContent = "";
    // A line in the file
    String line = "";
    int i = 1;
    while ((line = input.readLine()) != null) {
      // Add the line numbers
      newContent += i + ". " + line + "\n";
      i++;
    }
    // The output stream of the connection
    BufferedWriter output = new BufferedWriter(new OutputStreamWriter(connection.getOutputStream()));
    output.write(newContent);
    output.flush();
    //Close streams
    input.close();
    output.close();
  }

  /**
   * Test file2 protocol
   * @throws Exception 
   */
  public void testWriteInFile() throws Exception {
    //Create a new file in test dir
    File file = new File("test/file2.txt");
    FileWriter writer = new FileWriter(file);
    String content = "Method Summary\n"
        + "int available()\n"
        + "Returns the number of bytes that can be read from this file input stream without blocking.\n"
        + "void close()\n"
        + "Closes this file input stream and releases any system resources associated with the stream.\n"
        + "protected  void finalize()\n"
        + "Ensures that the close method of this file input stream is called when there are no more references to it.";
    //Write it's content
    writer.write(content);
    writer.flush();
    writer.close();

    // Install the protocols
    Installer.installProtocols();
    //Get the URL of the file using file2 protocol
    URL url = new URL("cproto:" + file.getAbsolutePath());
    
    //Test the protocol
    rewriteFile(url);
    
    //Check the new content of the file
    //The lines from the file must be numbered
    BufferedReader reader= new BufferedReader(new FileReader(file));
    String line = reader.readLine();
    assertTrue("content of the file not modified "+line, line.equals("1. Method Summary"));
    line = reader.readLine();
    assertTrue("content of the file not modified "+line, line.equals("2. int available()"));
    line = reader.readLine();
    assertTrue("content of the file not modified "+line, line.equals("3. Returns the number of bytes that can be read from this file input stream without blocking."));
    reader.close();
    
    //Delete the file
    file.delete();
  }
}
