/*
 *  The Syncro Soft SRL License
 *
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  3. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  4. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  5. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.sample.plugin.formwords;

import java.net.MalformedURLException;
import java.net.URL;

import junit.framework.TestCase;
import ro.sync.exml.plugin.selection.SelectionPluginContextImpl;

/**
 * @author iulian_velea
 * @created Feb 16, 2004
 * 
 */
public class FormWordsPluginExtensionTest extends TestCase {

  /**
   * Constructor for FormWordsPluginExtensionTest.
   * @param arg0
   */
  public FormWordsPluginExtensionTest(String arg0) {
    super(arg0);
  }

  /**
   * Test process method. 
   * 
   *@throws MalformedURLException If URL is malformed.
   */
  public void testProcess() throws MalformedURLException {
    String toTest = "  this is a sample.    this is a Sample   !";
    String result = "  This Is A Sample.    This Is A Sample   !";

    URL url = new URL("ftp://www.a.com/test/a.xml");

    SelectionPluginContextImpl pluginContext = new SelectionPluginContextImpl(toTest, null, url);
    FormWordsPluginExtension obj = new FormWordsPluginExtension();

    assertTrue(
      " Should be equal : " + obj.process(pluginContext).getProcessedSelection(),
      result.equals(obj.process(pluginContext).getProcessedSelection()));
    assertEquals(url, pluginContext.getDocumentURL());

    toTest = "tHis i\n" + 
             "s  <> # & ^   \n" + 
             "    ! aaa A sample.";
    result = "THis I\n" + 
             "S  <> # & ^   \n" + 
             "    ! Aaa A Sample.";
    pluginContext = new SelectionPluginContextImpl(toTest, null, url);
    assertEquals(url, pluginContext.getDocumentURL());

    assertTrue(
      " Should be equal : " + obj.process(pluginContext).getProcessedSelection(),
      result.equals(obj.process(pluginContext).getProcessedSelection()));
    
    toTest = "a\na";
    result = "A\nA";
    
    pluginContext = new SelectionPluginContextImpl(toTest, null, url);

    assertTrue(
        " Should be equal : " + obj.process(pluginContext).getProcessedSelection(),
        result.equals(obj.process(pluginContext).getProcessedSelection()));
    assertEquals(url, pluginContext.getDocumentURL());
  }
}
