/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.sample.plugin.workspace;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.text.BadLocationException;

import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.node.AuthorDocumentFragment;
import ro.sync.exml.editor.EditorPageConstants;
import ro.sync.exml.plugin.workspace.WorkspaceAccessPluginExtension;
import ro.sync.exml.workspace.api.editor.WSEditor;
import ro.sync.exml.workspace.api.editor.page.author.WSAuthorEditorPage;
import ro.sync.exml.workspace.api.listeners.WSEditorChangeListener;
import ro.sync.exml.workspace.api.standalone.MenuBarCustomizer;
import ro.sync.exml.workspace.api.standalone.StandalonePluginWorkspace;
import ro.sync.exml.workspace.api.standalone.ToolbarComponentsCustomizer;
import ro.sync.exml.workspace.api.standalone.ToolbarInfo;
import ro.sync.exml.workspace.api.standalone.ViewComponentCustomizer;
import ro.sync.exml.workspace.api.standalone.ViewInfo;
import ro.sync.ui.Icons;
import ro.sync.util.URLUtil;

/**
 * Plugin extension - workspace access extension.
 */
public class CustomWorkspaceAccessPluginExtension implements WorkspaceAccessPluginExtension {

  /**
   * Map between the URL of the temporary local file that contains the content of the 
   * checked out file and the URL of the checked out file.
   */
  private Map<URL, URL> openedCheckedOutUrls = new HashMap<URL, URL>();

  /**
   * If <code>true</code> the editor will be verified for Check In on close.  
   */
  private boolean verifyCheckInOnClose = true;
  
  /**
   * If <code>true</code> the document is Checked Out, it will be Checked In on 
   * close.  
   */
  private boolean forceCheckIn;

  /**
   * The CMS messages area.
   */
  private JTextArea cmsMessagesArea;

  /**
   * Plugin workspace access.
   */
  private StandalonePluginWorkspace pluginWorkspaceAccess;
  
  /**
   * If <code>true</code> then the plugin is running.
   */
  private boolean runPlugin;

  /**
   * @see ro.sync.exml.plugin.workspace.WorkspaceAccessPluginExtension#applicationStarted(ro.sync.exml.workspace.api.standalone.StandalonePluginWorkspace)
   */
  public void applicationStarted(final StandalonePluginWorkspace pluginWorkspaceAccess) {
    URL resource = getClass().getResource("/" + getClass().getName().replace('.', '/') + ".class");
    this.pluginWorkspaceAccess = pluginWorkspaceAccess;
    this.runPlugin = "jar".equals(resource.getProtocol()) ?  true : System.getProperty("cms.sample.plugin") != null; 
    if (runPlugin) {
      // Check In action
      final Action checkInAction = new AbstractAction() {
        public void actionPerformed(ActionEvent arg0) {
          WSEditor editorAccess = pluginWorkspaceAccess.getCurrentEditorAccess(StandalonePluginWorkspace.MAIN_EDITING_AREA);
          if (editorAccess != null) {
            URL tempFileUrl = editorAccess.getEditorLocation();
            // Get the corresponding URL
            URL checkedOutUrl = openedCheckedOutUrls.get(tempFileUrl);
            if (checkedOutUrl == null) {
              pluginWorkspaceAccess.showInformationMessage("The file is not Checked Out.");
            } else {
              checkInFile(pluginWorkspaceAccess, editorAccess, tempFileUrl, checkedOutUrl, true);
            }
          }
        }
      };

      // Check Out action 
      final Action checkOutAction = new AbstractAction() {
        public void actionPerformed(ActionEvent arg0) {
          File checkedOutFile = pluginWorkspaceAccess.chooseFile("Choose file", new String[] {"xml"}, "XML Files");
          if (checkedOutFile != null) {
            try {
              // Create temporary file from the checked out file.
              File tempFile = createTempFromCheckedOutFile(checkedOutFile);
              URL tempUrl = URLUtil.correct(tempFile);
              // Add the URls pair in the internal map 
              URL checkedoutUrl = URLUtil.correct(checkedOutFile);
              openedCheckedOutUrls.put(tempUrl, checkedoutUrl);

              // Open the temporary file
              pluginWorkspaceAccess.open(tempUrl);

              if (cmsMessagesArea != null) {
                String messages = cmsMessagesArea.getText() + "\n" + 
                "Check Out " + checkedoutUrl.toString();
                cmsMessagesArea.setText(messages);
              }
            } catch (Exception e) {
              pluginWorkspaceAccess.showErrorMessage("Check Out operation failed: " + e.getMessage());
            }
          }
        }
      };

      // Show Selection Source action
      final Action selectionSourceAction = new AbstractAction() {
        public void actionPerformed(ActionEvent actionevent) {
          WSEditor editorAccess = pluginWorkspaceAccess.getCurrentEditorAccess(StandalonePluginWorkspace.MAIN_EDITING_AREA);
          // The action is available only in Author mode.
          if (editorAccess != null && EditorPageConstants.PAGE_AUTHOR.equals(editorAccess.getCurrentPageID())) {
            WSAuthorEditorPage authorPageAccess = (WSAuthorEditorPage) editorAccess.getCurrentPage();
            AuthorDocumentController controller = authorPageAccess.getDocumentController();
            if (authorPageAccess.hasSelection()) {
              AuthorDocumentFragment selectionFragment;
              try {
                // Create fragment from selection
                selectionFragment = controller.createDocumentFragment(
                    authorPageAccess.getSelectionStart(),
                    authorPageAccess.getSelectionEnd() - 1
                );
                // Serialize
                String serializeFragmentToXML = controller.serializeFragmentToXML(selectionFragment);
                // Show fragment
                pluginWorkspaceAccess.showInformationMessage(serializeFragmentToXML);
              } catch (BadLocationException e) {
                pluginWorkspaceAccess.showErrorMessage("Show Selection Source operation failed: " + e.getMessage());
              }
            } else {
              // No selection
              pluginWorkspaceAccess.showInformationMessage("No selection available.");
            }
          }
        }
      };

      pluginWorkspaceAccess.addMenuBarCustomizer(new MenuBarCustomizer() {

        /**
         * @see ro.sync.exml.workspace.api.standalone.MenuBarCustomizer#customizeMainMenu(javax.swing.JMenuBar)
         */
        public void customizeMainMenu(JMenuBar mainMenuBar) {
          // CMS menu
          JMenu menuCMS = new JMenu("CMS"); 
          // Add the CMS menu before the Help menu
          mainMenuBar.add(menuCMS, mainMenuBar.getMenuCount() - 1);

          // Add Check In action on the menu 
          final JMenuItem checkInItem = new JMenuItem(checkInAction); 
          checkInItem.setText("Check In");
          menuCMS.add(checkInItem);

          // Add Check Out action on the menu
          JMenuItem checkOutItem = new JMenuItem(checkOutAction); 
          checkOutItem.setText("Check Out");
          menuCMS.add(checkOutItem);
          
          // Add Show Section Source action on the menu
          JMenuItem selectionSourceItem = new JMenuItem(selectionSourceAction);
          selectionSourceItem.setText("Show Selection Source");
          menuCMS.add(selectionSourceItem);
        }
      });

      pluginWorkspaceAccess.addEditorChangeListener(
          new WSEditorChangeListener() {

            @Override
            public void editorOpened(URL editorLocation) {
              checkActionsStatus(editorLocation);
            };

            // Check actions status
            private void checkActionsStatus(URL editorLocation) {
              WSEditor editorAccess = pluginWorkspaceAccess.getCurrentEditorAccess(StandalonePluginWorkspace.MAIN_EDITING_AREA);
              if (editorAccess != null) {
                selectionSourceAction.setEnabled(EditorPageConstants.PAGE_AUTHOR.equals(editorAccess.getCurrentPageID()));
              }
              URL checkedOutUrl = openedCheckedOutUrls.get(editorLocation);
              checkInAction.setEnabled(checkedOutUrl != null);
            }

            @Override
            public void editorClosed(URL editorLocation) {
              URL checkedOutUrl = openedCheckedOutUrls.get(editorLocation);
              if (checkedOutUrl != null) {
                if (verifyCheckInOnClose) {
                  if (forceCheckIn || pluginWorkspaceAccess.showConfirmDialog(
                      "Close", 
                      "The closed file " + editorLocation + " is Checked Out.\n Do you want to Check In?", 
                      new String[] {"Ok", "Cancel"}, 
                      new int[] {0, 1}) == 0) {            
                    // Save the current file.
                    checkInFile(pluginWorkspaceAccess, null, editorLocation, checkedOutUrl, false);
                  }
                }
                openedCheckedOutUrls.remove(editorLocation);
              }
            };

            @Override
            public void editorPageChanged(URL editorLocation) {
              checkActionsStatus(editorLocation);
            };

            @Override
            public void editorSelected(URL editorLocation) {
              checkActionsStatus(editorLocation);
            };
          }, 
          StandalonePluginWorkspace.MAIN_EDITING_AREA);


      pluginWorkspaceAccess.addToolbarComponentsCustomizer(new ToolbarComponentsCustomizer() {
        /**
         * @see ro.sync.exml.workspace.api.standalone.ToolbarComponentsCustomizer#customizeToolbar(ro.sync.exml.workspace.api.standalone.ToolbarInfo)
         */
        public void customizeToolbar(ToolbarInfo toolbarInfo) {
          if(ToolbarComponentsCustomizer.CUSTOM.equals(toolbarInfo.getToolbarID())) {
            // Check In
            JButton checkInButton = new JButton(checkInAction);
            checkInButton.setText("Check In");

            // Check Out
            JButton checkOutButton = new JButton(checkOutAction);
            checkOutButton.setText("Check Out");

            // Show Selection Source
            JButton selectionSourceButton = new JButton(selectionSourceAction);
            selectionSourceButton.setText("Show Selection Source");

            // Add in toolbar
            List<JComponent> comps = new ArrayList<JComponent>(); 
            comps.add(checkInButton);
            comps.add(checkOutButton);
            comps.add(selectionSourceButton);
            toolbarInfo.setComponents(comps.toArray(new JComponent[0]));

            // Set title
            toolbarInfo.setTitle("CMS");
          } 
        }
      });

      pluginWorkspaceAccess.addViewComponentCustomizer(new ViewComponentCustomizer() {
        /**
         * @see ro.sync.exml.workspace.api.standalone.ViewComponentCustomizer#customizeView(ro.sync.exml.workspace.api.standalone.ViewInfo)
         */
        public void customizeView(ViewInfo viewInfo) {
          if(ViewComponentCustomizer.CUSTOM.equals(viewInfo.getViewID())) {
            cmsMessagesArea = new JTextArea("CMS Session History:");
            viewInfo.setComponent(new JScrollPane(cmsMessagesArea));
            viewInfo.setTitle("CMS Messages");
            viewInfo.setIcon(Icons.CMS_MESSAGES_CUSTOM_VIEW);
          } else if("Project".equals(viewInfo.getViewID())) { 
            // Change the 'Project' view title.
            viewInfo.setTitle("CMS Project");
          } 
        }
      }); 
    }
  }

  /**
   * @see ro.sync.exml.plugin.workspace.WorkspaceAccessPluginExtension#applicationClosing()
   */
  public boolean applicationClosing() {
    if (runPlugin) {
      if (!openedCheckedOutUrls.isEmpty()) {
        int result = pluginWorkspaceAccess.showConfirmDialog(
            "Close", 
            "There are some opened Checked Out files.\n Do you want to Check In?", 
            new String[] {"Check In All", "Don't Check In", "Cancel"}, 
            new int[] {0, 1, 2});
        // Check In
        if (result == 0) {
          verifyCheckInOnClose = true;
          forceCheckIn = true;
        // Don't Check In
        } else if (result == 1) {
          verifyCheckInOnClose = false;
        // Cancel
        } else if (result == 2) {
          return false;
        }
      }
    }
    return true;
  }

  /**
   * Create temporary file from checked out file.
   * 
   * @param checkedOutFile The checked out file.
   * @return Temporary file.
   * @throws IOException
   * @throws FileNotFoundException
   */
  private File createTempFromCheckedOutFile(File checkedOutFile) throws IOException, FileNotFoundException {
    int indexOfPoint = checkedOutFile.getName().lastIndexOf('.');
    String fileName =  checkedOutFile.getName();
    // Temporary file name
    String tempFileName = indexOfPoint > -1 ? fileName.substring(0, indexOfPoint) : fileName;
    // Temporary file extension
    String fileExtension = indexOfPoint > -1 ? fileName.substring(indexOfPoint) : null;

    // Create temporary file
    File tempFile = File.createTempFile("cms_oxy" + tempFileName, fileExtension, checkedOutFile.getParentFile());

    // Write the content
    copyFileContent(checkedOutFile, tempFile);

    return tempFile;
  }

  /**
   * Copy content from one file to another.
   * 
   * @param initialFile The initial file to copy the content from.
   * @param destinationFile The destination.
   * @throws FileNotFoundException
   * @throws IOException
   */
  private void copyFileContent(File initialFile, File destinationFile) throws FileNotFoundException,
  IOException {
    FileInputStream fis = new FileInputStream(initialFile);
    FileOutputStream fos = new FileOutputStream(destinationFile);
    // Write the content
    int b;
    while( (b = fis.read()) != -1){
      fos.write(b);
    }
    fos.close();
    fis.close();
  }

  /**
   * Check In file.
   * 
   * @param pluginWorkspaceAccess The plugin workspace access
   * @param editorAccess The editor access.
   * @param tempFileUrl The temporary local file URL.
   * @param checkedOutUrl The URL of the corresponding checked out file
   */
  private void checkInFile(final StandalonePluginWorkspace pluginWorkspaceAccess,
      WSEditor editorAccess, URL tempFileUrl, URL checkedOutUrl, boolean openFile) {
    boolean checkIn = true; 
    // Verify if the editor is modified
    if (editorAccess != null && editorAccess.isModified()) {
      // Ask to save
      if (pluginWorkspaceAccess.showConfirmDialog(
          "Save", 
          "You must save the file in order to Check In.", 
          new String[] {"Ok", "Cancel"}, 
          new int[] {0, 1}) == 0) {            
        // Save the current file.
        editorAccess.save();
      } else {
        // Cancel.
        checkIn = false;
        pluginWorkspaceAccess.showInformationMessage("Check In operation was canceled.");
      }
    }

    // Perform Check In ...
    if (checkIn) {
      try {
        File tempFile = new File(tempFileUrl.getFile());

        // Copy the content of the temporary file over the original file
        copyFileContent(tempFile, new File(checkedOutUrl.getFile()));

        if (editorAccess != null) {
          // Close temporary file editor
          verifyCheckInOnClose = false;
          editorAccess.close(false);
          verifyCheckInOnClose = true;
        }

        // Delete temporary file
        tempFile.delete();

        if (openFile) {
          // Open the checked out file
          pluginWorkspaceAccess.open(checkedOutUrl);
          pluginWorkspaceAccess.showInformationMessage("Check In was performed.");
          if (cmsMessagesArea != null) {
            String messages = cmsMessagesArea.getText() + "\n" + 
            "Check In " + checkedOutUrl.toString();
            cmsMessagesArea.setText(messages);
          }
        }
      } catch (Exception e) {
        // Failed
        pluginWorkspaceAccess.showErrorMessage("Check In operation failed: " + e.getMessage()); 
      }
    }
  }
}